#include <LiquidCrystal.h>

// For RAMPS 1.4
// definitions taken from http://reprap.org/wiki/RAMPS_1.4
#define X_STEP_PIN         54
#define X_DIR_PIN          55
#define X_ENABLE_PIN       38
#define X_MIN_PIN           3
#define X_MAX_PIN           2

#define Y_STEP_PIN         60
#define Y_DIR_PIN          61
#define Y_ENABLE_PIN       56
#define Y_MIN_PIN          14
#define Y_MAX_PIN          15

#define Z_STEP_PIN         46
#define Z_DIR_PIN          48
#define Z_ENABLE_PIN       62
#define Z_MIN_PIN          18
#define Z_MAX_PIN          19

#define E_STEP_PIN         26
#define E_DIR_PIN          28
#define E_ENABLE_PIN       24

#define Q_STEP_PIN         36 // ??
#define Q_DIR_PIN          34 // not found on wiki page
#define Q_ENABLE_PIN       30

#define SDPOWER            -1
#define SDSS               53
#define LED_PIN            13

#define FAN_PIN            9

#define PS_ON_PIN          12

#define HEATER_0_PIN       10
#define HEATER_1_PIN       8
#define TEMP_0_PIN          13   // ANALOG NUMBERING
#define TEMP_1_PIN          14   // ANALOG NUMBERING

// LCD

#define KILL_PIN 41 //[RAMPS14-SMART-ADAPTER]

#define LCD_PINS_RS 16 //[RAMPS14-SMART-ADAPTER]
#define LCD_PINS_ENABLE 17 //[RAMPS14-SMART-ADAPTER]
#define LCD_PINS_D4 23 //[RAMPS14-SMART-ADAPTER]
#define LCD_PINS_D5 25 //[RAMPS14-SMART-ADAPTER]
#define LCD_PINS_D6 27 //[RAMPS14-SMART-ADAPTER]
#define LCD_PINS_D7 29 //[RAMPS14-SMART-ADAPTER]
#define LCD_PINS_RW 39 // set to -1 if not using lcdfast or no r/w

#define BTN_EN1 31 //[RAMPS14-SMART-ADAPTER]
#define BTN_EN2 33 //[RAMPS14-SMART-ADAPTER]
#define BTN_ENC 35 //[RAMPS14-SMART-ADAPTER]

#define BEEPER 37 //[RAMPS14-SMART-ADAPTER] / 37 = enabled; -1 = dissabled / (if you don't like the beep sound ;-)
#define SDCARDDETECT 49 //[RAMPS14-SMART-ADAPTER]

#define LCD_WIDTH 20
#define LCD_HEIGHT 4

LiquidCrystal lcd(LCD_PINS_RS, LCD_PINS_ENABLE, LCD_PINS_D4, LCD_PINS_D5, LCD_PINS_D6, LCD_PINS_D7);

void beep(int f=1000) {
int i;
boolean val = false;
for (i=1;i<100000/f;i++) {
  digitalWrite(BEEPER, val);
  val = val ^ true;
  delayMicroseconds(f);
  }
digitalWrite(BEEPER, LOW);
}


void setup() {
  pinMode(FAN_PIN , OUTPUT);
  pinMode(HEATER_0_PIN , OUTPUT);
  pinMode(HEATER_1_PIN , OUTPUT);
  pinMode(LED_PIN  , OUTPUT);
  pinMode(PS_ON_PIN, OUTPUT);
  
  pinMode(X_STEP_PIN  , OUTPUT);
  pinMode(X_DIR_PIN    , OUTPUT);
  pinMode(X_ENABLE_PIN    , OUTPUT);
  
  pinMode(Y_STEP_PIN  , OUTPUT);
  pinMode(Y_DIR_PIN    , OUTPUT);
  pinMode(Y_ENABLE_PIN    , OUTPUT);
  
  pinMode(Z_STEP_PIN  , OUTPUT);
  pinMode(Z_DIR_PIN    , OUTPUT);
  pinMode(Z_ENABLE_PIN    , OUTPUT);
  
  pinMode(E_STEP_PIN  , OUTPUT);
  pinMode(E_DIR_PIN    , OUTPUT);
  pinMode(E_ENABLE_PIN    , OUTPUT);
  
  pinMode(Q_STEP_PIN  , OUTPUT);
  pinMode(Q_DIR_PIN    , OUTPUT);
  pinMode(Q_ENABLE_PIN    , OUTPUT);
  
  pinMode(LCD_PINS_ENABLE, OUTPUT);
  pinMode(LCD_PINS_RS, OUTPUT);
  
  pinMode(BTN_ENC, INPUT_PULLUP);
  pinMode(BTN_EN1, INPUT_PULLUP);
  pinMode(BTN_EN2, INPUT_PULLUP);
  
  pinMode(KILL_PIN, INPUT_PULLUP);
  
  pinMode(X_MIN_PIN, INPUT); // endstops
  pinMode(X_MAX_PIN, INPUT);
  pinMode(Y_MIN_PIN, INPUT);
  pinMode(Y_MAX_PIN, INPUT);
  pinMode(Z_MIN_PIN, INPUT);
  pinMode(Z_MAX_PIN, INPUT);
  digitalWrite(X_MIN_PIN, HIGH);
  digitalWrite(X_MAX_PIN, HIGH);
  digitalWrite(Y_MIN_PIN, HIGH);
  digitalWrite(Y_MAX_PIN, HIGH);
  digitalWrite(Z_MIN_PIN, HIGH);
  digitalWrite(Z_MAX_PIN, HIGH);
  
  digitalWrite(PS_ON_PIN, HIGH); // power off after reset

  digitalWrite(X_ENABLE_PIN    , HIGH); // disable all Pololu's
  digitalWrite(Y_ENABLE_PIN    , HIGH);
  digitalWrite(Z_ENABLE_PIN    , HIGH);
  digitalWrite(E_ENABLE_PIN    , HIGH);
  digitalWrite(Q_ENABLE_PIN    , HIGH);

  lcd.begin(LCD_WIDTH, LCD_HEIGHT);
  lcd.clear();

  blink(20,100); // delay 2 sec before powering on
  digitalWrite(PS_ON_PIN, LOW); // power on!
  lcd.print("     POWER UP");

  blink(70,50); // delay 3.5 sec before powering on
  lcd.clear();
  lcd.print(" * 3D PRINTER TEST *       by AoG");
  
  delay(250);
  beep();
  beep(750);
  beep(500);
  beep(250);
  beep(125);
  beep(250);
  beep(500);
  beep(750);
  delay(300);
  
  updateDisplay(0,1000,0);
  
  digitalWrite(LED_PIN, HIGH); // main LED ON

}

void blink(int n, int d) { // flash n times with delay d
int i;
boolean val=0;
for (i=1;i<n;i++) {
  digitalWrite(LED_PIN, val);
  //digitalWrite(BEEPER, val);
  val = val ^ true;
  delay(d);
  }
}

void updateDisplay(int stepper,int speed,int dir) {
    lcd.home();
    lcd.print("   MOTOR: ");
    
    if (stepper == 0) lcd.print("X         ");
    if (stepper == 1) lcd.print("Y         ");
    if (stepper == 2) lcd.print("Z         ");
    if (stepper == 3) lcd.print("Extruder_0");
    if (stepper == 4) lcd.print("Extruder_1");

    lcd.print("   SPEED: ");
    lcd.print(2000/speed);
    lcd.print("  ");
    if (dir) lcd.print(">>"); else lcd.print("<<");
    lcd.print("   ");
 
}

void loop () {

  static int speed = 1000;
  static boolean run = true;
  static boolean step = LOW;
  static int stepper = 0;
  static boolean dir = LOW;
  static unsigned long prev_millis = 0;
  static unsigned long proc_counter = 0;

  unsigned long mls = micros()/speed;
  unsigned long mls1=millis();

  if (digitalRead(KILL_PIN) == LOW) {
    while (digitalRead(KILL_PIN) == LOW); // wait until released
    beep();
    while (digitalRead(KILL_PIN) == LOW);
    run = run ^ true;
    if (run) {
      blink(100,5);
      beep();
      updateDisplay(stepper,speed,dir);
      } else {
      lcd.home();
      lcd.print("       PAUSED                        ");
      }

    if (!run) {
       blink(100,5);
       beep();
       }

    }

  if (!run) {
// disable all steppers, turn off heaters and fan
    digitalWrite(X_ENABLE_PIN    , HIGH); // disable all Pololu's
    digitalWrite(Y_ENABLE_PIN    , HIGH);
    digitalWrite(Z_ENABLE_PIN    , HIGH);
    digitalWrite(E_ENABLE_PIN    , HIGH);
    digitalWrite(Q_ENABLE_PIN    , HIGH);

    digitalWrite(HEATER_0_PIN, LOW);
    digitalWrite(HEATER_1_PIN, LOW);
    digitalWrite(FAN_PIN, LOW);

              if (digitalRead(X_MAX_PIN) == LOW) {
                  lcd.clear();
                  lcd.print("    X_MAX trig");
                  beep(500);
                  blink(3,100);
                  delay(500);
                  lcd.home();
                  lcd.print("      PAUSED                         ");
                  }

              if (digitalRead(X_MIN_PIN) == LOW) {
                  lcd.clear();
                  lcd.print("    X_MIN trig");
                  beep(500);
                  blink(6,100);
                  delay(500);
                  lcd.home();
                  lcd.print("      PAUSED                         ");
                  }

              if (digitalRead(Y_MAX_PIN) == LOW) {
                  lcd.clear();
                  lcd.print("    Y_MAX trig");
                  beep(500);
                  blink(3,100);
                  delay(500);
                  lcd.home();
                  lcd.print("      PAUSED                         ");
                  }

              if (digitalRead(Y_MIN_PIN) == LOW) {
                  lcd.clear();
                  lcd.print("    Y_MIN trig");
                  beep(500);
                  blink(6,100);
                  delay(500);
                  lcd.home();
                  lcd.print("      PAUSED                         ");
                  }

                 if (digitalRead(Z_MAX_PIN) == LOW) {
                  lcd.clear();
                  lcd.print("    Z_MAX trig");
                  beep(500);
                  blink(3,100);
                  delay(500);
                  lcd.home();
                  lcd.print("      PAUSED                         ");
                  }

              if (digitalRead(Z_MIN_PIN) == LOW) {
                  lcd.clear();
                  lcd.print("    Z_MIN trig");
                  beep(500);
                  blink(6,100);
                  delay(500);
                  lcd.home();
                  lcd.print("      PAUSED                         ");
                  }


   
    return; // quit loop, don't do anything
    } else {
      
    }

  digitalWrite(LED_PIN,digitalRead(KILL_PIN));

  if (digitalRead(BTN_ENC) == LOW) {
    
    long counter = 0;
    while (digitalRead(BTN_ENC) == LOW) {
    counter++;
    if (counter>150000) {
        beep();  
        speed = speed / 2;
        if (speed < 70) speed=2000;
        

        while (digitalRead(BTN_ENC) == LOW); // wait until released
        beep(500);

        updateDisplay(stepper,speed,dir);

        return;
        }
    }

    beep();
    
    stepper++;    // rotate thru stepper motors
    if (stepper==5) stepper = 0;
    updateDisplay(stepper,speed,dir);

    }

  if (mls1 %1000 <300) { // flash heaters and fan LEDs
    digitalWrite(HEATER_0_PIN, HIGH);
    digitalWrite(HEATER_1_PIN, LOW);
    digitalWrite(FAN_PIN, LOW);

  } else if (mls1 %1000 <600) {
    digitalWrite(HEATER_0_PIN, LOW);
    digitalWrite(HEATER_1_PIN, HIGH);
    digitalWrite(FAN_PIN, LOW);
  } else  {
    digitalWrite(HEATER_0_PIN, LOW);
    digitalWrite(HEATER_1_PIN, LOW);
    digitalWrite(FAN_PIN, HIGH);
  }
  
  if (digitalRead(BTN_EN1)==LOW) {
    beep();
    beep(750);
    while (digitalRead(BTN_EN1)==LOW);
    dir = dir ^ true;
    
    updateDisplay(stepper,speed,dir);

    }

    if (mls != prev_millis) {

      proc_counter++;
      
      if (proc_counter == 50000) {
// auto-rotate thru motors
      beep(2000);
      beep(250);
      
      stepper++;    
      if (stepper==4) stepper = 0; // skip Q motor
      updateDisplay(stepper,speed,dir);
  
      proc_counter = 0;  
      }
      
      digitalWrite(X_DIR_PIN, dir); // set dir of all steppers
      digitalWrite(Y_DIR_PIN, dir); // 
      digitalWrite(Z_DIR_PIN, dir);
      digitalWrite(E_DIR_PIN, dir);
      digitalWrite(Q_DIR_PIN, dir);

      switch (stepper) {  // one step on selected stepper
      case 0: digitalWrite(X_STEP_PIN, step);

              digitalWrite(Y_ENABLE_PIN, HIGH);
              digitalWrite(Z_ENABLE_PIN, HIGH);
              digitalWrite(E_ENABLE_PIN, HIGH);
              digitalWrite(Q_ENABLE_PIN, HIGH);
              digitalWrite(X_ENABLE_PIN, LOW);

              if (digitalRead(X_MAX_PIN) == LOW && dir == HIGH) {
                  lcd.clear();
                  lcd.print("    X_MAX hit!");
                  dir = LOW;
                  beep(500);
                  blink(2,100);
                  delay(500);
                  updateDisplay(stepper,speed,dir);

                  }
              if (digitalRead(X_MIN_PIN) == LOW  && dir == LOW) {
                  lcd.clear();
                  lcd.print("    X_MIN hit!");
                  dir = HIGH;
                  beep();
                  blink(3,100);
                  beep();
                  blink(3,100);
                  updateDisplay(stepper,speed,dir);
                  }
              break;
      case 1: digitalWrite(Y_STEP_PIN, step);
              digitalWrite(X_ENABLE_PIN, HIGH);
              digitalWrite(Z_ENABLE_PIN, HIGH);
              digitalWrite(E_ENABLE_PIN, HIGH);
              digitalWrite(Q_ENABLE_PIN, HIGH);
              digitalWrite(Y_ENABLE_PIN, LOW);

              if (digitalRead(Y_MAX_PIN) == LOW  && dir == HIGH) {
                  lcd.clear();
                  lcd.print("    Y_MAX hit!");
                  dir = LOW;
                  beep(500);
                  blink(2,100);
                  delay(1000);
                  updateDisplay(stepper,speed,dir);
                  }
              if (digitalRead(Y_MIN_PIN) == LOW  && dir == LOW) {
                  lcd.clear();
                  lcd.print("    Y_MIN hit!");
                  dir = HIGH;
                  beep();
                  blink(3,100);
                  beep();
                  blink(3,100);
                  updateDisplay(stepper,speed,dir);
                  }
              break;
      case 2: digitalWrite(Z_STEP_PIN, step);
              digitalWrite(X_ENABLE_PIN, HIGH);
              digitalWrite(Y_ENABLE_PIN, HIGH);
              digitalWrite(E_ENABLE_PIN, HIGH);
              digitalWrite(Q_ENABLE_PIN, HIGH);
              digitalWrite(Z_ENABLE_PIN, LOW);
              if (digitalRead(Z_MAX_PIN) == LOW  && dir == HIGH) {
                  lcd.clear();
                  lcd.print("    Z_MAX hit!");
                  dir = LOW;
                  beep(500);
                  blink(2,100);
                  delay(1000);
                  updateDisplay(stepper,speed,dir);
                  }
              if (digitalRead(Z_MIN_PIN) == LOW  && dir == LOW) {
                  lcd.clear();
                  lcd.print("    Z_MIN hit!");
                  dir = HIGH;
                  beep();
                  blink(3,100);
                  beep();
                  blink(3,100);
                  updateDisplay(stepper,speed,dir);
                  }
              break;

      case 3: digitalWrite(E_STEP_PIN, step);
              digitalWrite(X_ENABLE_PIN, HIGH);
              digitalWrite(Y_ENABLE_PIN, HIGH);
              digitalWrite(Z_ENABLE_PIN, HIGH);
              digitalWrite(Q_ENABLE_PIN, HIGH);
              digitalWrite(E_ENABLE_PIN, LOW);
              break;

      case 4: digitalWrite(Q_STEP_PIN, step);
              digitalWrite(X_ENABLE_PIN, HIGH);
              digitalWrite(Y_ENABLE_PIN, HIGH);
              digitalWrite(Z_ENABLE_PIN, HIGH);
              digitalWrite(E_ENABLE_PIN, HIGH);
              digitalWrite(Q_ENABLE_PIN, HIGH);
              digitalWrite(Q_ENABLE_PIN, LOW);
              break;
      }  

      step = step ^ true;
      }
 
    prev_millis = mls;
  
}
